/**
  ******************************************************************************
  * @file      ex_rtc.h
  * @author    Marek Laban, Marcel Kleja, MICRONIC a.s., LabHC
  * @version   v1.0
  * @note      Created in frame of HECTOR project (No. 644052).
  * @copyright MICRONIC a.s.,
  * @date      11-December-2016
  * @revision  v1.1 (ML)
  *             - Initial version
  * @brief     Fundamental functions for communication with external RTC timer
  *
  ******************************************************************************
  */

#ifndef EX_RTC_H_
#define EX_RTC_H_

#include <stdint.h>

/* Public functions */
void set_rtc_time (uint8_t *rtc_array);
void get_rtc_time (uint8_t rtc_array[6]);
void init_rtc     (void);
void set_fattime  (uint32_t time_word);
uint8_t is_rtc_init (void);

// RTC array size
#define RTC_SIZE 6


// RTC state
#define RTC_READY  1
#define RTC_NREADY 0

// RTC function variables
#define RTC_YR 0     // year
#define RTC_MO 1     // month
#define RTC_DY 2     // day
#define RTC_HR 3     // hour
#define RTC_MI 4     // minute
#define RTC_SC 5     // second

// RTC register address
#define RTC_REG_MS   0x00 // miliseconds
#define RTC_REG_SC   0x01 // seconds
#define RTC_REG_MI   0x02 // minutes
#define RTC_REG_HR   0x03 // hours
#define RTC_REG_DW   0x04 // day of week
#define RTC_REG_DY   0x05 // day of month
#define RTC_REG_MO   0x06 // month
#define RTC_REG_YR   0x07 // year

#endif /* EX_RTC_H_ */
