---------------------------------------------------------------------
--
-- Design unit:   Package of constants for vhdl modules
--
-- File name:     mb_pkg.vhd
--
-- Description:   This package defines common types, subtypes etc.
--
-- System:        VHDL'93
--
-- Autor:         Marek Laban, Viktor Fischer, Ivan Pecar
--
-- Copyright:     Hubert Curien Laboratory
--
-- Revision:      Version 7.01, May 2017
--
-- History:       29/05/2017 - v7.01 Version of motherboard included
--
--
---------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;

PACKAGE mb_pkg IS
  
  -- Motherboard version
  CONSTANT MB_VER      : STD_LOGIC_VECTOR(3 DOWNTO 0) := X"B";
  -- Motherboard subversion
  CONSTANT MB_SBVER    : STD_LOGIC_VECTOR(3 DOWNTO 0) := X"0";
 
--------------------------------------------------------------------------
-- Definitions of commands for the control block  
--------------------------------------------------------------------------
  -- Idle mode (nothing to do)
  CONSTANT CMD_OP_IDLE    : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"00000000";
  -- Operation start !! fixed for MSS !!
  CONSTANT CMD_OP_ST      : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"00000001";
  -- Set daughter board 1 active (HDMI) 
  CONSTANT CMD_SETDB1     : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"00000002";
  -- Set daughter board 2 active (SATA) 
  CONSTANT CMD_SETDB2     : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"00000003"; 
  -- Get TRNG status
  CONSTANT CMD_SSI_LOAD_SEND_LOW : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"00000006"; 
  -- User command 1 (not used)
  CONSTANT CMD_GET_TIME      : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"00000007";  
  -- Load high 32 bits of SSI word
  CONSTANT CMD_SSI_LOAD_HIGH : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"00000008";
  -- Read high 32 bits from SSI word
  CONSTANT CMD_READ_SSI_HIGH : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"00000009";
  -- Read low 32 bits from SSI word
  CONSTANT CMD_READ_SSI_LOW  : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"0000000A"; 
  -- start crypto operation
  CONSTANT CMD_CRYPTO_ENA    : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"0000000B"; 
  -- set constant N
  CONSTANT SET_N             : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"0000000C"; 

  -- Start Address of available RAM memory restricted for FPGA
  CONSTANT RAM_ADR_START     : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"A0000000"; 
  -- End address of restricted memory for FPGA
  CONSTANT RAM_ADR_END       : STD_LOGIC_VECTOR(31 DOWNTO 0) := X"A2000000";
  --
 

END mb_pkg;